﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Collections;
using System.Globalization;

namespace Esempio5
{
    static class OperationsDescriptorsProvider
    {
        public static IEnumerable<IOperationsDescriptor> GetOperationsDescriptors()
        {
            return from type in Assembly.GetExecutingAssembly().GetTypes()
                   where type.GetInterface(typeof(IOperationsDescriptor).Name) != null
                   where !type.IsAbstract
                   where type.GetConstructor(Type.EmptyTypes) != null
                   select (IOperationsDescriptor) Activator.CreateInstance(type);
        }

        class BaseOperationsByName : IOperationsDescriptor
        {
            public virtual string MenuDescription
            {
                get { return "Base operations by name"; }
            }

            public virtual string DisplayMember
            {
                get { return "Name"; }
            }

            public virtual bool Diadic
            {
                get { return true; }
            }

            public virtual IEnumerable GetOperations()
            {
                List<OperationBase> operations = new List<OperationBase>();
                foreach (Type type in Assembly.GetExecutingAssembly().GetTypes())
                {
                    //if (type.BaseType == typeof(OperationBase))
                    if (type.IsSubclassOf(typeof(OperationBase)))
                    {
                        if (type.IsAbstract)
                        {
                            throw new ApplicationException(String.Format("Il tipo {0} è astratto.", type));
                        }
                        else if (type.GetConstructor(Type.EmptyTypes) == null)
                        {
                            throw new ApplicationException(String.Format("Il tipo {0} non contiene il costruttore di default.", type));
                        }
                        else
                        {
                            OperationBase operation = (OperationBase) Activator.CreateInstance(type);
                            operations.Add(operation);
                        }
                    }
                }
                return operations;
            }

            public virtual string GetResult(object operation, string operationName, string leftOperand, string rightOperand)
            {
                double operando1 = Double.Parse(leftOperand, NumberStyles.Float);
                double operando2 = Double.Parse(rightOperand, NumberStyles.Float);
                OperationBase baseOperation = (OperationBase) operation;
                return String.Format("{0} {1} {2} = {3}",
                    operando1, operationName, operando2, baseOperation.Evaluate(operando1, operando2));
            }
        }

        class BaseOperationsBySymbolicName : BaseOperationsByName
        {
            public override string MenuDescription
            {
                get { return "Base operations by symbolic name"; }
            }

            public override string DisplayMember
            {
                get { return "SymbolicName"; }
            }
        }

        class MathOperationsByName : IOperationsDescriptor
        {
            public virtual string MenuDescription
            {
                get { return "Math operations by name"; }
            }

            public virtual string DisplayMember
            {
                get { return "Name"; }
            }

            public virtual bool Diadic
            {
                get { return false; }
            }

            public virtual IEnumerable GetOperations()
            {
                List<MethodInfo> operations = new List<MethodInfo>();
                foreach (MethodInfo methodInfo in typeof(Math).GetMethods())
                {
                    ParameterInfo[] parameterInfos = methodInfo.GetParameters();
                    if (parameterInfos.Length == 1 && parameterInfos[0].ParameterType == typeof(double) && methodInfo.ReturnType == typeof(double))
                    {
                        operations.Add(methodInfo);
                    }
                }
                return operations;
            }

            public virtual string GetResult(object operation, string operationName, string leftOperand, string rightOperand)
            {
                double operando = Double.Parse(rightOperand, NumberStyles.Float);
                MethodInfo mathOperation = (MethodInfo) operation;
                return String.Format("{0}({1}) = {2}",
                    operationName, operando, mathOperation.Invoke(null, new object[] { operando }));
            }
        }
    }
}
